package com.vincenzojuniorforte.poliutodroid.model;

/**
 * GUI Event class
 * <p>
 * It represents an event that triggers a state change. A GuiEvent object contains details about
 * the interested GUI states (i.e., source and target), the action performed and the widget on which
 * the input has been injected.
 */
public class GuiEvent {
    //region Variables
    private GuiState parentGuiState;
    private GuiWidget triggeredWidget;
    private GuiWidget.Action action;
    private GuiState arrivalGuiState;
    //endregion

    //region Constructors

    /**
     * GuiEvent constructor
     *
     * @param parentGuiState  GuiState in which the event has been performed
     * @param triggeredWidget GuiWidget in which the event has been performed
     * @param action          the action performed
     */
    public GuiEvent(GuiState parentGuiState, GuiWidget triggeredWidget, GuiWidget.Action action) {
        this.parentGuiState = parentGuiState;
        this.triggeredWidget = triggeredWidget;
        this.action = action;
    }
    //endregion

    //region Getters and Setters

    /**
     * Get the GuiState in which the event has been performed
     *
     * @return the GuiState in which the event has been performed
     */
    public GuiState getParentGuiState() {
        return parentGuiState;
    }

    /**
     * Get the GuiWidget in which the event has been performed
     *
     * @return the GuiWidget in which the event has been performed
     */
    public GuiWidget getTriggeredWidget() {
        return triggeredWidget;
    }

    /**
     * Get the action related to the event
     *
     * @return the action related to the event
     */
    public GuiWidget.Action getAction() {
        return action;
    }

    /**
     * Get the GuiState where the event led
     *
     * @return the GuiState where the event led
     */
    public GuiState getArrivalGuiState() {
        return arrivalGuiState;
    }

    /**
     * Set the GuiState where the event led
     *
     * @param arrivalGuiState the GuiState where the event led
     */
    public void setArrivalGuiState(GuiState arrivalGuiState) {
        this.arrivalGuiState = arrivalGuiState;
    }
    //endregion

    //region Overrides and Implementations
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        GuiEvent guiEvent = (GuiEvent) o;

        if (getParentGuiState() != null ? !getParentGuiState().equals(guiEvent.getParentGuiState()) : guiEvent.getParentGuiState() != null)
            return false;
        if (getTriggeredWidget() != null ? !getTriggeredWidget().equals(guiEvent.getTriggeredWidget()) : guiEvent.getTriggeredWidget() != null)
            return false;
        if (getAction() != guiEvent.getAction()) return false;
        return getArrivalGuiState() != null ? getArrivalGuiState().equals(guiEvent.getArrivalGuiState()) : guiEvent.getArrivalGuiState() == null;

    }

    @Override
    public int hashCode() {
        int result = getParentGuiState() != null ? getParentGuiState().hashCode() : 0;
        result = 31 * result + (getTriggeredWidget() != null ? getTriggeredWidget().hashCode() : 0);
        result = 31 * result + (getAction() != null ? getAction().hashCode() : 0);
        result = 31 * result + (getArrivalGuiState() != null ? getArrivalGuiState().hashCode() : 0);
        return result;
    }
    //endregion
}
