package com.vincenzojuniorforte.poliutodroid.util;

import android.os.SystemClock;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Random;

/**
 * Utility class
 * <p>
 * It provides useful functions.
 */
public class Utility {

    /**
     * Sleep for a while
     *
     * @param min minimum time to sleep before returning, in milliseconds of uptime.
     * @param max maximum time to sleep before returning, in milliseconds of uptime.
     */
    public static void randomDelay(float min, float max) {
        int random = randomInt(min, max);
        SystemClock.sleep(random);
    }

    /**
     * Return a random integer within a given range
     *
     * @param min minimum value of the range
     * @param max maximum value of the range
     * @return random integer within the range
     */
    public static int randomInt(float min, float max) {
        return (int) (max * Math.random() + min);
    }

    /**
     * Return a random integer within a given range
     *
     * @param p1 one bound of the range
     * @param p2 other bound of the range
     * @return random integer within the range
     */
    public static int randomIntBounded(int p1, int p2) {
        int lowerLimit, upperLimit;

        if (p1 > p2) {
            upperLimit = p1;
            lowerLimit = p2;
        } else {
            upperLimit = p2;
            lowerLimit = p1;
        }

        return lowerLimit + (int) (new Random().nextFloat() * (upperLimit - lowerLimit));
    }

    /**
     * Execute a command printing outputs
     *
     * @param str_cmd command to be executed
     * @throws IOException
     */
    public static void executeCommandLineWithOutputPrinting(String str_cmd) throws IOException {
        Process process = Runtime.getRuntime().exec(str_cmd);

        BufferedReader stdInput = new BufferedReader(new
                InputStreamReader(process.getInputStream()));

        BufferedReader stdError = new BufferedReader(new
                InputStreamReader(process.getErrorStream()));

        // Read the output from the command
        System.out.println("Here is the standard output of the command:\n");
        String s = null;
        while ((s = stdInput.readLine()) != null) {
            System.out.println(s);
        }

        // Read any errors from the attempted command
        System.out.println("Here is the standard error of the command (if any):\n");
        while ((s = stdError.readLine()) != null) {
            System.out.println(s);
        }
    }

    /**
     * Execute a command returning outputs
     *
     * @param str_cmd command to be executed
     * @return the output of the command or null if some error occurred
     * @throws IOException
     */
    public static String executeCommandLineForResults(String str_cmd) throws IOException {
        Process process = Runtime.getRuntime().exec(str_cmd);

        BufferedReader stdError = new BufferedReader(new
                InputStreamReader(process.getErrorStream()));

        // Errors occurred
        if (stdError.readLine() == null) {
            return null;
        }

        BufferedReader stdInput = new BufferedReader(new
                InputStreamReader(process.getInputStream()));

        // Return output from the command
        return stdInput.toString();
    }
}
